// ItaliaDaRicostruire
// by Massimo Avvisati under GPL v.3


import processing.pdf.*;

import pbox2d.*;
import org.jbox2d.collision.shapes.*;
import org.jbox2d.common.*;
import org.jbox2d.dynamics.*;

// A reference to our box2d world
PBox2D box2d;

boolean saveOneFrame = false;


// A list we'll use to track fixed objects
ArrayList<Boundary> boundaries;
// A list for all of our rectangles
ArrayList<Box> boxes;

PFont ubuntu;
int currentFontSize = 42;

PImage bg;

void setup() {
  bg = loadImage("italia.png");
  size(bg.width, bg.height, P2D);
  //scene = new Scene(this);
  colorMode(HSB, 360, 100, 100);

  ubuntu = loadFont("ubuntu_font.vlw");

  textFont(ubuntu, currentFontSize);
  textAlign(CENTER, CENTER);

  smooth();

  // Initialize box2d physics and create the world
  box2d = new PBox2D(this);
  box2d.createWorld();
  // We are setting a custom gravity
  box2d.setGravity(0, 0);

  // Create ArrayLists	
  boxes = new ArrayList<Box>();
  boundaries = new ArrayList<Boundary>();

  // Add a bunch of fixed boundaries
  boundaries.add(new Boundary(width/4, height-5, width/2-50, 10));
  boundaries.add(new Boundary(3*width/4, height-50, width/2-50, 10));
}

void draw() {
  //  background(bg);
  background(#470CC9);
  // We must always step through time!
  box2d.step();

  // Boxes fall from the top every so often
  /*if (random(1) < 0.2) {
   
   Box p = new Box(width/2,30);
   boxes.add(p);
   }
   
   if (mousePressed && isMap(mouseX, mouseY)) {
   Box p = new Box(mouseX, mouseY);//, random(30, 40), random(10, 20));
   boxes.add(p);
   println("ah");
   
   }
   
   // Display all the boundaries
   for (Boundary wall: boundaries) {
   wall.display();
   }*/
  if (saveOneFrame == true) {
    beginRecord(PDF, "immagini_create/" + millis() + ".pdf");
  }
  // Display all the boxes
  for (Box b: boxes) {
    b.display();
  }

  if (saveOneFrame == true) {
    endRecord();
    saveOneFrame = false;
  }

  // Boxes that leave the screen, we delete them
  // (note they have to be deleted from both the box2d world and our list
  for (int i = boxes.size()-1; i >= 0; i--) {
    Box b = boxes.get(i);
    if (b.done()) {
      boxes.remove(i);
    }
  }
  if (mustSaveImage) {
    save("immagini_create/" + millis() + ".png"); 
    mustSaveImage = false;
  }
}
boolean mustSaveImage = false;
String[] messages = new String[] {
  "PACE", "TOLLERANZA", "LAVORO", "CITTADINANZA", "BIMBI", "GIOCO", "DIRITTI", "SPIRITUALITA'", "VOLONTARIATO", "LIBERTA'", "AMORE", "AMICIZIA", "SOLIDARIETA'", "NON VIOLENZA"
    , "GENEROSITA'", "VERITA'", "ONESTA'", "CREATIVITA'", "UMILTA'", "EDUCAZIONE", "GENTILEZZA", "RISPETTO", "SINCERITA'", "VITA", "LAICITA'", "SPIRITUALITA'", "CULTURA", "CIVILTA'"
};

//
int wordCounter = 0;
String getNextWord() {
  return messages[wordCounter++ % messages.length];
}

int colorAngle = 0;

void keyPressed() {
  if (key == 'i')
    mustSaveImage = true;
  if (key == 'p')
    saveOneFrame = true;
}

void mouseClicked() {
  colorMode(HSB, 720, 100, 100);
  if (mouseButton == LEFT) {
    for (int i = 0; i < 300; i++) {
      int x = int(random(width));
      int y = int(random(height));
      if (isMap(x, y)) {
        textFont(ubuntu, currentFontSize);
        WordBox p = new WordBox(getNextWord(), x, y, color(colorAngle, 100, 100), random(0, 360));
        boxes.add(p);
        currentFontSize = int(random(8, 10));
        colorAngle += 3;
        colorAngle %= 360;

        println("ah");
      }
    }

    for (int i = 0; i < 200; i++) {
      int x = int(random(width));
      int y = int(random(height));
      if (isMap(x, y)) {
        textFont(ubuntu, currentFontSize);
        WordBox p = new WordBox(getNextWord(), x, y, color(colorAngle, 100, 100), random(0, 360));
        boxes.add(p);
        currentFontSize = int(random(16, 32));
        colorAngle += 3;
        colorAngle %= 360;

        println("ah");
      }
    }
  } 
}

boolean isMap(int x, int y) {
  return (saturation(bg.get(x, y)) > 0);
}



// The Nature of Code
// <http://www.shiffman.net/teaching/nature>
// Spring 2012
// PBox2D example

// A fixed boundary class

class Boundary {

  // A boundary is a simple rectangle with x,y,width,and height
  float x;
  float y;
  float w;
  float h;
  
  // But we also have to make a body for box2d to know about it
  Body b;

  Boundary(float x_,float y_, float w_, float h_) {
    x = x_;
    y = y_;
    w = w_;
    h = h_;

    // Define the polygon
    PolygonShape sd = new PolygonShape();
    // Figure out the box2d coordinates
    float box2dW = box2d.scalarPixelsToWorld(w/2);
    float box2dH = box2d.scalarPixelsToWorld(h/2);
    // We're just a box
    sd.setAsBox(box2dW, box2dH);


    // Create the body
    BodyDef bd = new BodyDef();
    bd.type = BodyType.STATIC;
    bd.position.set(box2d.coordPixelsToWorld(x,y));
    b = box2d.createBody(bd);
    
    // Attached the shape to the body using a Fixture
    b.createFixture(sd,1);
  }

  // Draw the boundary, if it were at an angle we'd have to do something fancier
  void display() {
    /*
    fill(0);
    stroke(0);
    rectMode(CENTER);
    rect(x,y,w,h);
    */
  }

}

// A rectangular box
class Box {

  // We need to keep track of a Body and a width and height
  Body body;
  float w;
  float h;

  // Constructor
 
 Box(float x, float y, float w, float h, float angle) {
    this.w = w;
    this.h = h;
    makeBody(new Vec2(x, y), w, h, angle);
  }
  // This function removes the particle from the box2d world
  void killBody() {
    box2d.destroyBody(body);
  }

  // Is the particle ready for deletion?
  boolean done() {
    // Let's find the screen position of the particle
    Vec2 pos = box2d.getBodyPixelCoord(body);
    // Is it off the bottom of the screen?
    if (pos.y > height+w*h) {
      killBody();
      return true;
    }
    return false;
  }

  // Drawing the box
  void display() {
    // We look at each body and get its screen position
    Vec2 pos = box2d.getBodyPixelCoord(body);
    // Get its angle of rotation
    float a = body.getAngle();

    rectMode(CENTER);
    pushMatrix();
    translate(pos.x, pos.y);
    rotate(-a);
    fill(175);
    stroke(0);
    rect(0, 0, w, h);
    popMatrix();
  }

  // This function adds the rectangle to the box2d world
  void makeBody(Vec2 center, float w_, float h_, float angle) {

    // Define a polygon (this is what we use for a rectangle)
    PolygonShape sd = new PolygonShape();
    float box2dW = box2d.scalarPixelsToWorld(w_/2);
    float box2dH = box2d.scalarPixelsToWorld(h_/2);
    sd.setAsBox(box2dW, box2dH);

    // Define a fixture
    FixtureDef fd = new FixtureDef();
    fd.shape = sd;
    // Parameters that affect physics
    fd.density = 1;
    fd.friction = 0.3;
    fd.restitution = 0.5;

    // Define the body and make it from the shape
    BodyDef bd = new BodyDef();
    bd.type = BodyType.DYNAMIC;
    bd.position.set(box2d.coordPixelsToWorld(center));

    body = box2d.createBody(bd);
    body.createFixture(fd);
    body.setTransform(body.getPosition(), radians(angle));

    // Give it some initial random velocity
    /*
    body.setLinearVelocity(new Vec2(random(-5, 5), random(2, 5)));
    body.setAngularVelocity(random(-5, 5));
    */
  }
}

class WordBox extends Box {
  String message;
  int fontSize = 32;
  color fillColor;
  WordBox(String message, float x, float y, color fillColor, float angle) {
    super(x, y, textWidth(message), currentFontSize, angle);
    this.message = message;
    this.fontSize = currentFontSize;
    this.fillColor = fillColor;
  }
  
   // Drawing the box
   @Override
  void display() {
    // We look at each body and get its screen position
    Vec2 pos = box2d.getBodyPixelCoord(body);
    // Get its angle of rotation
    float a = body.getAngle();

    rectMode(CENTER);
    pushMatrix();
    translate(pos.x, pos.y);
    rotate(-a);
   /*
    noFill();
    stroke(0);
    rect(0, 0, w, h);
    */
    fill(fillColor);
    noStroke();
    textFont(ubuntu, fontSize);
    text(message, 0, 0);
    
    popMatrix();
  }
}

